#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 02/18/2001
#################################################################################
#
# Board - Messages board functionality for Ethereal-Realms
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2E 0K9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Date;						# Date processing
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table creation


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my %param;							# Parameter hash
my %board;							# Board hash

#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Database::GetHashBoard($Database, \%board);	# Retreive hash

	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	Webchat::Database::DocumentGetHeader($Database);	# HTML header

	if ( Webchat::Login::GetVerificationNormal($Database, $cgi, \%param) )
	{
		if (RealmSelect($Database, $cgi, \%param, \%board))
		{
			# Adjust title
			$board{'TagTitle'} =~ s/REALM/$param{'ROOM'}/gs;

			# Display title
			print "$board{'TagTitle'}\n";

			# Logic handling
			# Writer
			if (defined($param{'WRITE'}))
			{ PostWrite($Database, $cgi, \%param, \%board); }

			# Composer
			if ((defined($param{'REPLY'})) || (defined($param{'POST'})))
			{ PostComposer($Database, $cgi, \%param, \%board); }

			# Posts viewer (default)
			else
			{ PostView($Database, $cgi, \%param, \%board); }
		}
	}

	Webchat::Database::DocumentGetFooter($Database);	# HTML footer
	$Database->disconnect();				# Disconnect database handle

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Realm Selection 
#
# Realms are needed in order to complete operations for the message board.  This
# is for the primary reason that the board grants one topic area for each specific
# realm.  Without a public realm, the board topic will not exist.

sub RealmSelect 						# Database, CGI, Parameter list, Board hash
{
	#####################
	# Data members
	my $Database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $board    = shift;					# Board hash handle 

	my $realms;						# Composit of realms
	my @realms;						# Realm selection


	#####################
	# Program area

	# Display only if needed
	unless (defined($param->{'ROOM'}))
	{
		#Retreive realm list
		Webchat::Database::GetListPublicRealms($Database, \@realms);

		# Format line for use
		$realms = $cgi->scrolling_list('ROOM', \@realms, $realms[0], 5);

		# Display Realm Prompt
		# Beginning of form
		print $cgi->start_form();

		# Display Table
		Webchat::Table::MakeTop($cgi);
		Webchat::Table::MakeValid($cgi, $board->{'TagRealm'}, $realms);
		Webchat::Table::MakeBlank($cgi);
		Webchat::Table::MakeValid($cgi, $cgi->submit($board->{'TxtSelect'}), $cgi->reset($board->{'TxtReset'}));
		Webchat::Table::MakeBottom($cgi);

		# End of form
		Webchat::Param::EmbedNormal($Database, $cgi, %{$param});
		print $cgi->end_form(), "\n";

		# Return false
		return 0;
	}

	# Return if defined
	return 1;

}

#####################
# Post Viewer 
#
# The viewing sub-routine for the message board.  Of course the specific realm is needed
# which is why that this is called after the ROOM is determined. 

sub PostView 						# Database, CGI, Parameter list, Board hash 
{
	#####################
	# Data members
	my $Database = shift;				# Database handle
	my $Statement;					# Query handle

	my $cgi      = shift;			 	# CGI handle	
	my $param    = shift;				# Parameter list
	my $board    = shift;				# Board hash

	my $left;					# Rows left of entry
	my $right;					# Rows for entry

	my $inline;					# Inline parameter list
	my $url      = $cgi->url();			# Self referencing link

	my $rows     = 10;				# Complete set of rows
	my $count    = 0;				# Rows retreived

	my $res;					# Results hash


	#####################
	# Program area

	# Generate parameter lines
	my $post = Webchat::Param::EmbedInline($Database, $cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'POST', 'True');
	my $back = Webchat::Param::EmbedInline($Database, $cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});


	# Prepare and execute
	$Statement = $Database->prepare("SELECT BoardIDNumber,
			 PuppetName,
			 BoardSubject,
			 BoardTimestamp,
			 BoardMessage,
			 BoardLevel,
			 LevelZero,
			 LevelOne,
			 LevelTwo,
			 LevelThree,
			 LevelFour,
			 LevelFive
			FROM  Board 
			WHERE RealmName=?
			ORDER BY 
			 LevelZero,
			 LevelOne,
			 LevelTwo,
			 LevelThree,
			 LevelFour,
			 LevelFive"); 
	$Statement->execute($param->{'ROOM'});

	
	# HTML Display
	print "<BR>\n\n<CENTER><TABLE WIDTH=\"95%\" CELLSPACING=\"1\" CELLPADDING=\"2\">\n";

	# Status view
	print "\t<TR BGCOLOR=\"$board->{SetColorHead}\">",
	      "\t\t<TD COLSPAN=\"$rows\"><A HREF=\"$url$post\">$board->{TagPost}</A> ",
      	      "<A HREF=\"$url$back\">$board->{TagBack}</A></TD>\n",
	      "\t</TR>\n";

	# Retreive and display
	while ($res = $Statement->fetchrow_hashref())
	{
		# Increment counter
		$count++;

		# Determine rows
		$right = $rows - $res->{'BoardLevel'};
		$left  = $rows - $right;

		# Generate inline
		$inline = Webchat::Param::EmbedInline($Database, $cgi, %{$param}, 'REPLY', $res->{'BoardIDNumber'});


		# Header
		print "\t<TR>\n";

		# Handling based on level
		if ($res->{'BoardLevel'} != 0)		# Children of threads
		{
			print "\t\t<TD COLSPAN=\"$left\">&nbsp;</TD>\n";
			print "\t\t<TD COLSPAN=\"$right\" BGCOLOR=\"$board->{SetColorHead}\">",
		      	      "<FONT COLOR=\"$board->{SetColorText}\">$board->{TagHeadSubject}$res->{BoardSubject}</FONT><BR>",
			      "<FONT COLOR=\"$board->{SetColorText}\">$board->{TagHeadFrom}$res->{PuppetName}",
			      " $board->{TagHeadDate} ", Webchat::Date::GetDate($res->{'BoardTimestamp'}), "</FONT><BR>",
			      "</TD>\n";
		}
		else					# Threads parent
		{
			print "\t\t<TD COLSPAN=\"$rows\" BGCOLOR=\"$board->{SetColorHead}\">",
		      	      "<FONT COLOR=\"$board->{SetColorText}\">$board->{TagHeadSubject}$res->{BoardSubject}</FONT><BR>",
			      "<FONT COLOR=\"$board->{SetColorText}\">$board->{TagHeadFrom}$res->{PuppetName}",
			      " $board->{TagHeadDate} ", Webchat::Date::GetDate($res->{'BoardTimestamp'}), "</FONT><BR>",
			      "</TD>\n";
		}

		print "\t</TR>\n";


		# Message
		print "\t<TR>\n";

		# Handling based on level
		if ($res->{'BoardLevel'} != 0)		# Children of threads
		{
			print "\t\t<TD COLSPAN=\"$left\">&nbsp;</TD>\n";
			print "\t\t<TD COLSPAN=\"$right\" BGCOLOR=\"$board->{SetColorBody}\">",
			      "<FONT COLOR=\"$board->{SetColorText}\">$res->{BoardMessage}</FONT>",
			      "<BR><A HREF=\"$url$inline\">$board->{TagReply}</A>",
			      "</TD>\n";
		}
		else					# Threads parent
		{
			print "\t\t<TD COLSPAN=\"$rows\" BGCOLOR=\"$board->{SetColorBody}\">",
			      "<FONT COLOR=\"$board->{SetColorText}\">$res->{BoardMessage}</FONT>",
			      "<BR><A HREF=\"$url$inline\">$board->{TagReply}</A>",
			      "</TD>\n";
		}
	
		print "\t</TR>\n";
	}

	# Status view
	print "\t<TR BGCOLOR=\"$board->{SetColorHead}\">",
	      "\t\t<TD COLSPAN=\"$rows\">$board->{TagCount} $count<BR>",
	      "<A HREF=\"$url$post\">$board->{TagPost}</A> ",
      	      "<A HREF=\"$url$back\">$board->{TagBack}</A></TD>\n",
	      "\t</TR>\n";


	# Spacer row
	print "\t<TR>\n";
	for (my $i; $i < $rows; $i++) { print "\t\t<TD>&nbsp;</TD>\n"; } 

	# End of table
	print "\t</TR>\n</TABLE></CENTER>\n";


	# Close query handle
	$Statement->finish();
}


#####################
# Post Composer  
#
# As the name would imply, this subroutine allows for the addition of posts to the 
# existing set of posts.  Provided that there are posts to reply to, one can
# reply under the parent post.

sub PostComposer 						# Database, CGI, Parameter list
{
	#####################
	# Data members

	my $Database = shift; 					# Database handle
	my $Statement;						# Query handle

	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $board    = shift;					# Board hash handle 

	my $subj;						# Subject of message

	my $puppets;						# Prepared output
	my @puppets;						# List of puppets


	#####################
	# Program area
	
	# Gather information
 	if (defined($param->{'REPLY'}))
	{
		# Prepare and execute
		$Statement = $Database->prepare("SELECT BoardSubject FROM Board WHERE BoardIDNumber=?");
		$Statement->execute($param->{'REPLY'});

		# Retrieve and finish
		($subj) = $Statement->fetchrow();
		$Statement->finish();

		# Adapt subject
		if ($subj !~ /^\(RE\)/)
		{ $subj =~ s/(.+)/(RE) $1/; }

		# Assign ID for next pass
		$param->{'PID'} = $param->{'REPLY'};
	}

	# Cleanup
	delete $param->{'REPLY'};
	delete $param->{'POST'};


	# Puppet list 
	# Retreive puppet list
	Webchat::Database::GetListPuppetYours($Database, \@puppets, $param->{'USER'});

	# Determine puppets
	$puppets = $cgi->popup_menu('PFROM', \@puppets, $puppets[0]);


	# Display area
	# Display title
	print "$board->{'TagTitlePost'}\n";

	# Start of form, table
	print $cgi->start_form();
	Webchat::Table::MakeTop($cgi);

	# Header
	Webchat::Table::MakeSingle($cgi, $board->{'TagPostHead'});
	Webchat::Table::MakeValid($cgi, $board->{'TagPostFrom'}, $puppets);
	Webchat::Table::MakeValid($cgi, $board->{'TagPostSubject'}, $cgi->textfield('PSUBJ', $subj,25,250));

	# Body
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeSingle($cgi, $board->{'TagPostMessage'});
	Webchat::Table::MakeSingle($cgi, "<TEXTAREA NAME=\"PTEXT\", ROWS=\"10\", COLS=\"60\", WRAP=\"SOFT\"></TEXTAREA>");


	# Submit and clear
	Webchat::Table::MakeBlank($cgi);
	Webchat::Table::MakeValid($cgi, $cgi->submit($board->{'TxtPost'}), $cgi->reset($board->{'TxtReset'}));

	Webchat::Table::MakeBottom($cgi);
	Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'WRITE', 'True');
	print $cgi->end_form(), "\n";

}


#####################
# Post Writer  
#
# Handles the eventual cleanup and posting of the post to the threads and tree.  This includes
# language filtering and removal of HTML to keep this clean.
	
sub PostWrite	 						# Database, CGI, Parameter list
{
	#####################
	# Data members

	my $Database = shift; 					# Database handle
	my $Statement;						# Query handle

	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $board    = shift;					# Board hash handle

	my $puppet;						# Puppet name
	my $puppeteer;						# Puppeteer name

	my $level;						# Level
	my $res;						# Results

	my $id  = 'LAST_INSERT_ID()';				# ID insertion function
	my $url = $cgi->url();					# Location to self

	my @levels;						# Levels and depth
	my %system;						# System hash


	#####################
	# Program area

	# Information handling
	# Retreive post information
	if (defined($param->{'PID'}))
	{
		# Prepare and execute
		$Statement = $Database->prepare("SELECT PuppetName,
				 BoardLevel,
				 LevelZero,
				 LevelOne,
				 LevelTwo,
				 LevelThree,
				 LevelFour,
				 LevelFive
				FROM  Board
				WHERE BoardIDNumber=?");
		$Statement->execute($param->{'PID'});

		# Retreive and adapt
		$res = $Statement->fetchrow_hashref();

		# Determine levels
		$levels[0] = $res->{'LevelZero'};
		$levels[1] = ($res->{'LevelOne'}   != $res->{'LevelZero'})  ? $res->{'LevelOne'}   : $id;
		$levels[2] = ($res->{'LevelTwo'}   != $res->{'LevelOne'})   ? $res->{'LevelTwo'}   : $id;
		$levels[3] = ($res->{'LevelThree'} != $res->{'LevelTwo'})   ? $res->{'LevelThree'} : $id;
		$levels[4] = ($res->{'LevelFour'}  != $res->{'LevelThree'}) ? $res->{'LevelFour'}  : $id;
		$levels[5] = ($res->{'LevelFive'}  != $res->{'LevelFour'})  ? $res->{'LevelFive'}  : $id;

		# Determin response level
		$level = $res->{'BoardLevel'} + 1;
		$level = ($level > 5) ? 5 : $level; 

		# Determin puppet name
		$puppet = $res->{'PuppetName'};


		# Finish query
		$Statement->finish();
	}
	
	# Parent (root level) posts
	else
	{
		# Assign levels and level for root
		@levels = ($id, $id, $id, $id, $id, $id); 
		$level = 0; 
	}


	# Mold subject and body
	# Strip HTML
	$param->{'PSUBJ'} =~ s/<[^>]*>//gs;
	$param->{'PTEXT'} =~ s/<[^>]*>//gs;

	# Replace carriage returns
	$param->{'PTEXT'} =~ s/\n/<BR>/gs;


	# Database insertion
	$Database->do("INSERT INTO Board 
		(BoardIDNumber,
		 PuppetName,
		 RealmName,
		 BoardSubject,
		 BoardTimestamp,
		 BoardMessage,
		 BoardLevel,
		 LevelZero,
		 LevelOne,
		 LevelTwo,
		 LevelThree,
		 LevelFour,
		 LevelFive)
		VALUES
		(NULL,?,?,?,?,?,?,0,0,0,0,0,0)", {},

	 $param->{'PFROM'},
	 $param->{'ROOM'},
	 $param->{'PSUBJ'},
	 time,
	 $param->{'PTEXT'},
	 $level);

	# MySQL cludge
	$Database->do("UPDATE Board
		SET LevelZero=$levels[0],
		    LevelOne=$levels[1],
		    LevelTwo=$levels[2],
		    LevelThree=$levels[3],
		    LevelFour=$levels[4],
		    LevelFive=$levels[5]
		WHERE BoardIDNumber=LAST_INSERT_ID()", {});


	# Email notification
	if (defined($puppet))
	{
		# Retreive login
		($puppeteer) = Webchat::Database::DataGetPuppeteerLogin($Database, $puppet);


		# Complete if defined
		if (defined($puppeteer))
		{
			# Retreive name and Email
			my ($name)   = Webchat::Database::DataGetPuppeteerName($Database, $puppeteer);
			my ($email)  = Webchat::Database::DataGetPuppeteerEmail($Database, $puppeteer);
			my ($notice) = Webchat::Database::DocumentGetBoardNotice($Database);

			if (defined($notice))
			{
				# Link with hash
				Webchat::Database::GetHashSystem($Database, \%system);

 				# Go though all keywords and replace with real value
 				$notice =~ s/FULLNAME/$name/g;
 				$notice =~ s/ORIGIN/$param->{'PFROM'}/g;
				$notice =~ s/REALM/$param->{'ROOM'}/g;
 				$notice =~ s/RETURNEMAIL/$system{'SetInfoContactAddress'}/g;
 				$notice =~ s/SERVERNAME/$system{'SetInfoServer'}/g;
 				$notice =~ s/SUBJECT/$param->{'PSUBJ'}/g;
 				$notice =~ s/URL/$url/g;


				# Define and open sendmail
				open(SENDMAIL, "|$system{'LocSendmail'} -oi -t")
   				   or die("Cannot fork sendmail: $!");

				# Send out invitation
				print SENDMAIL "From: $system{'SetInfoContactName'} <$system{'SetInfoContactAddress'}>\n";
				print SENDMAIL "To: $name <$email>\n";
				print SENDMAIL "Subject: You have a reply on $param->{ROOM} message board from: $param->{'PFROM'} \n\n";
				print SENDMAIL "$notice\n";
				close(SENDMAIL);
			}
		}
	}


	# Cleanup
	delete $param->{'PID'};
	delete $param->{'PFROM'};
	delete $param->{'PSUBJ'};
	delete $param->{'PTEXT'};
	delete $param->{'WRITE'};
}
